//
// MoonCanvas.cpp
// Draw the moon and related info.
//
//    Copyright 2002 by Akkana Peck.
// You may use or distribute this code under the terms of the GPL.
//

#include "MoonCanvas.h"

#include "Crater.h"

#include <qpainter.h>
#include <qimage.h>
#include <qtextview.h>

// XXX next two should be removable:
#include <stdio.h>
#include <math.h>
#include <values.h>    // for MAXINT
#include <stdlib.h>

MoonCanvas::MoonCanvas( QWidget *parent, const char *name )
    : SkyCanvas( parent, name ),
      moonRadius(0), moonCenterX(-1), moonCenterY(-1), mMirrored(1)
{
}

MoonCanvas::~MoonCanvas()
{
    if (mPainter)
        delete mPainter;
    if (moonImg)
        delete moonImg;
}

void MoonCanvas::Init()
{
    moonCenterX = width()/2;
    moonCenterY = moonCenterX;

    moonImg = new QImage (QString("/opt/Qtopia/share/qmoon/fullmoon.png"));
    moonRadius = moonImg->width()/2 - 4;

    setMinimumHeight(width());

    mPainter = new QPainter(this);

    Crater::ReadCraterFile("/opt/Qtopia/share/qmoon/qmoon.dat", moonRadius);
}

void MoonCanvas::ToggleMirrored()
{
    mMirrored = -mMirrored;

    moonImg = new QImage (QString(mMirrored > 0
                                ? "/opt/Qtopia/share/qmoon/fullmoon.png"
                                : "/opt/Qtopia/share/qmoon/fullmoon-mirror.png"
                              ));
    moonRadius = moonImg->width()/2 - 4;
    Paint();
}

QSize MoonCanvas::minimumSizeHint() const
{
    return QSize(width(), width());
}

void MoonCanvas::Paint()
{
    if (moonRadius <= 0)
        Init();
    if (!mPainter)
        return;

    Clear();
    mPainter->drawImage(moonCenterX-moonImg->width()/2,
                        moonCenterY-moonImg->height()/2,
                        *moonImg);
    mCraterNum = 0;

    return;
}

void MoonCanvas::PaintCrater(Crater* crater)
{
    mPainter->setPen(QColor(255, 50, 0));

    mPainter->drawEllipse(moonCenterX + crater->GetX() * mMirrored
                           - crater->GetSizeX()/2,
                          moonCenterY + crater->GetY()
                           - crater->GetSizeY()/2,
                          crater->GetSizeX(), crater->GetSizeY());

    // Label the crater's name
    mPainter->setBackgroundMode(QPainter::OpaqueMode);
    //mPainter->setBackgroundColor(QColor("white"));
    mPainter->setBackgroundColor(QColor(255, 255, 255));
    mPainter->drawText(moonCenterX + crater->GetX() * mMirrored
                        - crater->GetSizeX()/2,
                       moonCenterY + crater->GetY() - crater->GetSizeY()/2 - 3,
                       QString(crater->GetName()));

    // And more information
    mPainter->setBackgroundMode(QPainter::TransparentMode);
    mPainter->setPen(QColor(255, 255, 255));
    QString numstr;
    numstr.setNum(crater->GetSize());
    static int textHeight = 14;
    int starty = 242 + textHeight * 2 * mCraterNum;
    QRect bounds;
    mPainter->drawText(5, starty, 228, textHeight, Qt::AlignLeft,
                       QString(crater->GetName()) + QString(" (")
                        + numstr + QString(" km)"),
                       -1, &bounds);
    textHeight = bounds.height();
    mPainter->drawText(10, starty + textHeight,
                       228, textHeight, Qt::AlignLeft,
                       QString(crater->GetDesc()));
    ++mCraterNum;
}

void MoonCanvas::mouseReleaseEvent(QMouseEvent *event)
{
    Paint();

    double fx = (event->pos().x() - moonCenterX) * mMirrored;
    double fy = event->pos().y() - moonCenterY;

    // Translate that to moon coordinates
    double fLatitude = -asin(fy / moonRadius);
    double latitude = fLatitude * 180./M_PI;
    double longitude = asin(fx / moonRadius / cos(fLatitude)) * 180./M_PI;

    // Figure out which crater lives there
    double mindist = moonRadius*moonRadius;
    Crater* nearcrater = 0;
    Crater* crater;
    Crater::InitLoop();
    while ((crater = Crater::NextCrater()) != 0)
    {
        int isInside;
        double thisdist = crater->Distance(longitude, latitude, &isInside);
        if (isInside)
        {
            mindist = 0;
            nearcrater = 0;
            PaintCrater(crater);
        }
        else if (thisdist < mindist)
        {
            nearcrater = crater;
            mindist = thisdist;
        }
    }

    // Draw the nearby crater if we aren't inside of one:
    if (nearcrater)
    {
        //printf("Landed at %s\n", nearcrater->GetName());
        PaintCrater(nearcrater);
    }
}

#if 0
QSizePolicy MoonCanvas::sizePolicy() const
{
    //return QSizePolicy( QSizePolicy::Expanding, QSizePolicy::Fixed );
    QSizePolicy policy(QSizePolicy::Expanding, QSizePolicy::Fixed);
    policy.
    return policy;
}
#endif
